/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) ProLogic, Incorporated 2008, 1000 Technology Drive, Fairmont, WV 26554
 * Produced pursuant to U.S. Government sponsorship.
 * 
 * Copyright applies to commercial uses only.   Notwithstanding the above
 * copyright/restriction, the Government possesses DFARS 252.227-7013. 
 * Unlimited Rights if this is technical data and DFARS 252.227-7014. 
 * Unlimited Rights if this is computer software or computer software 
 * documentation.
 * 
 * $Id: $
 *****************************************************************************/
/**
 * @file
 * Defines a byte vector w/ a static maximum user-defined size.
 */

///////////////////////////////////////////////////////////////////////////////
// include guard
///////////////////////////////////////////////////////////////////////////////
#pragma once



///////////////////////////////////////////////////////////////////////////////
// includes
///////////////////////////////////////////////////////////////////////////////
#include <memory>
#include <stddef.h> //for size_t



//some forward declarations so we're not including headers needlessly
namespace std
{
	template<class _Elem, class _Traits, class _Ax> class basic_string;
	template<class _Ty,	class _Ax> class vector;
}



/**
 * Custom allocator.  It's simply the std::allocator but with a different max_size()
 * so containers won't grow beyond a staticly defined limit.  This isn't normally
 * used directly but instead is passed as a template parameter of an STL container.
 *
 * Inherits publicly from std::allocator to expose its members but be aware that 
 * std::allocator does not provide a virtual destructor.  This shouldn't be a problem
 * for the STL since derivation from std::allocator is neither required nor assumed 
 * and allocators aren't used on the heap.
 *
 * Example usage:
 *
 *   vector<unsigned char, MaxAllocator<968, unsigned char> > vecKey; //key, max 968 bytes long
 *   basic_string<char, char_traits<char>, MaxAllocator<24, char> > strDest; //destination, max 24 chars long
 */
template<size_t _nMaxSize, class _T>
class MaxAllocator : public std::allocator<_T>
{
private:
	typedef MaxAllocator<_nMaxSize, _T> MyType;
	typedef std::allocator<_T> BaseType;

public:
	enum { nMaxSize = _nMaxSize }; //<Maximum array size

	using typename BaseType::value_type;      //<Element type
	using typename BaseType::pointer;         //<Pointer to element
	using typename BaseType::reference;       //<Reference to element
	using typename BaseType::const_pointer;   //<Constant pointer to element
	using typename BaseType::const_reference; //<Constant reference to element
	using typename BaseType::size_type;       //<Quantities of elements
	using typename BaseType::difference_type; //<Difference between two pointers

	//@todo Several members below have not been needed so far but their implementation
	//is important.  They've been commented out until we can force their use (in a unit
	//test or other controlled condition) and work out any issues.  See <xmemory> on 
	//Windows and http://www.cplusplus.com/reference/std/memory/allocator/ for reference

	/*
	 * Constructors
	 */

	///Construct default allocator
	MaxAllocator() throw()
	{
	}

	//@note see todo above for why this is commented out
	//Construct by copying
	//MaxAllocator(const MaxAllocator<_nMaxSize, _T>& rhs) throw()
	//: BaseType(rhs)
	//{
	//}

	//@note see todo above for why this is commented out
	//Construct from a related allocator
	//template<size_t _nOtherSize, class _OtherT>
	//MaxAllocator(const MaxAllocator<_nOtherSize, _OtherT>& rhs) throw()
	//: BaseType(rhs)
	//{
	//}

	//@note see todo above for why this is commented out
	//Construct from a related allocator
	//template<class _OtherT>
	//MaxAllocator(const std::allocator<_OtherT>& rhs) throw()
	//: BaseType(rhs)
	//{
	//}

	/*
	 * Member functions
	 */

	//@note see todo above for why this is commented out
	//Assign from a related allocator
	//template<size_t _nOtherSize, class _OtherT>
	//MaxAllocator<_nMaxSize, _T>& operator=(const MaxAllocator<_nOtherSize, _OtherT>& rhs)
	//{
	//	m_al = rhs.m_al;
	//	return *this;
	//}

	//@note see todo above for why this is commented out
	//Assign from a related allocator
	//template<class _OtherT>
	//MaxAllocator<_nMaxSize, _T>& operator=(const std::allocator<_OtherT>& rhs)
	//{
	//	BaseType::operator=(rhs);
	//	return *this;
	//}

private:
	///Make a private assignment to prevent a default one; remove after
	///a real one is created
	//@note see todo above for why this is here 
	template<class _OtherT>
	MaxAllocator<_nMaxSize, _T>& operator=(const std::allocator<_OtherT>& rhs)
	{
		BaseType::operator=(rhs);
		return *this;
	}
public:

	//\return Maximum size possible to allocate
	size_type max_size() const throw()
	{
		return nMaxSize;
	}
};




/**
 * Defines typedefs for common containers so using MaxAllocator is not so painful.
 *
 * Example usage:
 *
 *   MaxAlloc<968>::bytevec vecKey; //key, max 968 bytes long
 *
 * The resulting vector is just a normal STL vector of unsigned char, but
 * with a custom allocator that sets a non-default max size.  As will happen
 * when you max out a default vector, this one will throw a length_error.
 * This one just throws a whole lot sooner.
 */
template<size_t _nMaxSize>
class MaxAlloc
{
public:
	///Maximum array size / maximum number of elements
	enum { nSize = _nMaxSize };

	///Vector of unsigned char w/ a user-defined max capacity
	typedef std::vector<unsigned char, MaxAllocator<nSize, unsigned char> > bytevec;

	///String w/ a user-defined max capacity
	typedef std::basic_string<char, std::char_traits<char>, MaxAllocator<nSize, char> > string;
};

