/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file SubjectRo.h
 * Main header file for the PROJECT_NAME application.
 * 
 * @todo Fix file description.
 */


#pragma once

//Wraps CSubject to make the subject Read-Only.
//
//Works by adding a layer of indirection which does not include methods to modify 
//the underlying value or send out updates.  In order to modify the value and/or 
//send out updates to the registered observers you need access to the CSubject 
//variable that's passed into the ctors.  Update *that* CSubject & the Observers
//will get Updated.
//
//This way an interface can expose Read-Only Subjects & keep Set & Update access
//private.
//
//The wrappers are very thin and should "compile-away" to be equivalent to
//accessing the CSubject directly.

#include "Subject.h"



template<typename T>
struct CSubjectRo
{
	//init the subject
	CSubjectRo( CSubject<T>& subj )
	:	m_subj(subj)
	{
	}


	//gets the value
	const T& Get() const
	{
		return m_subj.Get();
	}


	//registers a callback method
	template<class Observer>
	void Attach( Observer& observer, void(Observer::*pMethod)(const T&))
	{
		m_subj.Attach<Observer>( observer, pMethod );
	}


	//UN-registers a callback method
	template<class Observer>
	void Detach( Observer& observer, void(Observer::*pMethod)(const T&))
	{
		m_subj.Detach<Observer>( observer, pMethod );
	}

private:
	CSubject<T>& m_subj;
};




///Specialization for reference types.
///Minor differences in the way const qualifiers are handled.
template<typename T>
struct CSubjectRo<T&>
{
	//init the subject
	CSubjectRo( CSubject<T&>& subj )
	:	m_subj(subj)
	{
	}


	//gets the value
	const T& Get() const
	{
		return m_subj.Get();
	}


	//registers a callback method
	template<class Observer>
	void Attach( Observer& observer, void(Observer::*pMethod)(const T&))
	{
		m_subj.Attach<Observer>( observer, pMethod );
	}


	//UN-registers a callback method
	template<class Observer>
	void Detach( Observer& observer, void(Observer::*pMethod)(const T&))
	{
		m_subj.Detach<Observer>( observer, pMethod );
	}

private:
	CSubject<T&>& m_subj;
};




///Specialization for type void.
///Basically turns CSubjectRo into a triggering mechanism
template<>
struct CSubjectRo<void>
{
	//init the subject
	CSubjectRo( CSubject<void>& subj )
	:	m_subj(subj)
	{
	}


	// registers a callback method
	template<class Observer>
	void Attach( Observer& observer, void(Observer::*pMethod)())
	{
		m_subj.Attach<Observer>( observer, pMethod );
	}


	// UN-registers a callback method
	template<class Observer>
	void Detach( Observer& observer, void(Observer::*pMethod)())
	{
		m_subj.Detach<Observer>( observer, pMethod );
	}

private:
	CSubject<void>& m_subj;
};



