#include "FieldTest.h"
#include "Field.h"
#include "CompileTimeCheck.h"
#include "TestUtils.h"
#include "boost/cstdint.hpp"
#include <vector>
#include <list>
#include <iterator>

using namespace std;
using namespace boost;


//compares 2 arrays for equality
template<typename T, size_t n>
static bool Equal( const T (&lhs)[n], const T (&rhs)[n] )
{
	for( size_t i = 0; i < n; i++ )
	{
		if( lhs[i] != rhs[i] )
		{
			return false;
		}
	}
	return true;
}


//prints an array to an ostream
template<typename T, size_t n>
static ostream& operator<<( ostream& o, const T (&rhs)[n] )
{
	o << '[';
	for( size_t i = 0; i < n-1; i++ )
	{
		o << rhs[i] << ',';
	}
	o << rhs[n-1] << ']';
	return o;
}


//inserts data into an array
template<typename Buff, typename T>
static void CopyWithPointer( T tData )
{
	//calc how big our array needs to be, but don't let it be zero
	enum
	{
		_div = sizeof T / sizeof Buff,
		nCount = (_div > 0 ? _div : 1)
	};

	//expected output array; byte-copy t into it
	Buff aExpected[nCount];
	memcpy( aExpected, &tData, sizeof tData ); //by defn, sizeof tData <= sizeof aExpected
	STATIC_CHECK( sizeof tData <= sizeof aExpected, Array_Too_Small );

	//copy to a destination buffer
	Buff aBuff[nCount];
	Buff* const pNext = Field::Copy( aBuff, aBuff + nCount, tData );

	//make sure a copy into a too-small buffer throws
	CPPUNIT_ASSERT_THROW( Field::Copy( aBuff, aBuff + nCount -1, tData ), std::invalid_argument );

	//assert they're equal
	CPPUNIT_ASSERT( Equal( aExpected, aBuff ));

	//make sure our next insertion point follows the array
	CPPUNIT_ASSERT_EQUAL( aBuff + nCount, pNext );
}


void CFieldTest::CopyWithPointerTest()
{
	//purposely use signed buffers, to catch signing issues

	//test storing into byte-buffer
	CopyWithPointer<int8_t, int8_t>( -0x11 );
	CopyWithPointer<int8_t, uint8_t>( -0x11 );
	CopyWithPointer<int8_t, int16_t>( -0x1122 );
	CopyWithPointer<int8_t, uint16_t>( -0x1122 );
	CopyWithPointer<int8_t, int32_t>( -0x11223344 );
	CopyWithPointer<int8_t, uint32_t>( -0x11223344 );
	CopyWithPointer<int8_t, int64_t>( -0x1122334455667788 );
	CopyWithPointer<int8_t, uint64_t>( -0x1122334455667788 );
	CopyWithPointer<int8_t, float>( (float)123.456 );
	CopyWithPointer<int8_t, double>( 123.456 );
	CopyWithPointer<int8_t, long double>( 123.456 );

	//test storing into 2-byte-buffer
	CopyWithPointer<int16_t, int16_t>( 0x1122 );
	CopyWithPointer<int16_t, uint16_t>( 0x1122 );
	CopyWithPointer<int16_t, int32_t>( 0x11223344 );
	CopyWithPointer<int16_t, uint32_t>( 0x11223344 );
	CopyWithPointer<int16_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int16_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int16_t, float>( (float)123.456 );
	CopyWithPointer<int16_t, double>( 123.456 );
	CopyWithPointer<int16_t, long double>( 123.456 );

	//test storing into 4-byte-buffer
	CopyWithPointer<int32_t, int32_t>( 0x11223344 );
	CopyWithPointer<int32_t, uint32_t>( 0x11223344 );
	CopyWithPointer<int32_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int32_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int32_t, float>( (float)123.456 );
	CopyWithPointer<int32_t, double>( 123.456 );
	CopyWithPointer<int32_t, long double>( 123.456 );

	//test storing into 8-byte-buffer
	CopyWithPointer<int64_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int64_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int64_t, double>( 123.456 );
	CopyWithPointer<int64_t, long double>( 123.456 );

	//by design, these cases fail to compile
	//CopyWithPointer<int16_t, int8_t>( -0x11 );
	//CopyWithPointer<int32_t, int8_t>( -0x11 );
	//CopyWithPointer<int32_t, int16_t>( -0x11 );
}




//inserts data into a container using an iterator
template<class Cont, typename T>
static void CopyWithIterator( T tData )
{
	typedef Cont::value_type Entry; //what the container holds
	enum { nCount = sizeof T / sizeof Entry };

	//copy data into the container
	Cont container( nCount ); //pre-allocate entries so we don't overrun
	Cont::iterator iNext = Field::Copy( container.begin(), container.end(), tData );

	//make sure a copy into a too-small buffer throws
	CPPUNIT_ASSERT_THROW( Field::Copy( container.begin(), container.end() -1, tData ), std::invalid_argument );

	//assert we've got the right number of elements in the container
	CPPUNIT_ASSERT_EQUAL((size_t)nCount, container.size());

	//assert the contents are what we expect them to be
	const Cont expected((const Entry*)&tData, (const Entry*)(&tData+1));
	CPPUNIT_ASSERT_EQUAL( expected, container );

	//make sure our next insertion point is at the end of the container
	CPPUNIT_ASSERT( container.end() == iNext );
}



void CFieldTest::CopyWithIteratorTest()
{
	//purposely use signed buffers, to catch signing issues

	//test storing into byte-buffer
	CopyWithIterator<vector<int8_t>, int8_t>( -0x11 );
	CopyWithIterator<vector<int8_t>, uint8_t>( -0x11 );
	CopyWithIterator<vector<int8_t>, int16_t>( -0x1122 );
	CopyWithIterator<vector<int8_t>, uint16_t>( -0x1122 );
	CopyWithIterator<vector<int8_t>, int32_t>( -0x11223344 );
	CopyWithIterator<vector<int8_t>, uint32_t>( -0x11223344 );
	CopyWithIterator<vector<int8_t>, int64_t>( -0x1122334455667788 );
	CopyWithIterator<vector<int8_t>, uint64_t>( -0x1122334455667788 );
	CopyWithIterator<vector<int8_t>, float>( (float)123.456 );
	CopyWithIterator<vector<int8_t>, double>( 123.456 );
	CopyWithIterator<vector<int8_t>, long double>( 123.456 );

	//test storing into 2-byte-buffer
	CopyWithIterator<vector<int16_t>, int16_t>( 0x1122 );
	CopyWithIterator<vector<int16_t>, uint16_t>( 0x1122 );
	CopyWithIterator<vector<int16_t>, int32_t>( 0x11223344 );
	CopyWithIterator<vector<int16_t>, uint32_t>( 0x11223344 );
	CopyWithIterator<vector<int16_t>, int64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int16_t>, uint64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int16_t>, float>( (float)123.456 );
	CopyWithIterator<vector<int16_t>, double>( 123.456 );
	CopyWithIterator<vector<int16_t>, long double>( 123.456 );

	//test storing into 4-byte-buffer
	CopyWithIterator<vector<int32_t>, int32_t>( 0x11223344 );
	CopyWithIterator<vector<int32_t>, uint32_t>( 0x11223344 );
	CopyWithIterator<vector<int32_t>, int64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int32_t>, uint64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int32_t>, float>( (float)123.456 );
	CopyWithIterator<vector<int32_t>, double>( 123.456 );
	CopyWithIterator<vector<int32_t>, long double>( 123.456 );

	//test storing into 8-byte-buffer
	CopyWithIterator<vector<int64_t>, int64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int64_t>, uint64_t>( 0x1122334455667788 );
	CopyWithIterator<vector<int64_t>, double>( 123.456 );
	CopyWithIterator<vector<int64_t>, long double>( 123.456 );

	//by design, these cases fail to compile
	//CopyWithIterator<vector<int16_t>, int8_t>( -0x11 );
	//CopyWithIterator<vector<int32_t>, int8_t>( -0x11 );
	//CopyWithIterator<vector<int32_t>, int16_t>( -0x11 );
}




template<class Cont, typename T>
static void InsertToContainer( T tData )
{
	typedef Cont::value_type Entry; //what the container holds

	//copy data into the container
	Cont container;
	Field::Insert( back_inserter(container), tData );

	//assert we've got the right number of elements in the container
	CPPUNIT_ASSERT_EQUAL( sizeof T/sizeof Entry, container.size());

	//assert the contents are what we expect them to be
	const Cont expected((const Entry*)&tData, (const Entry*)(&tData+1));
	CPPUNIT_ASSERT_EQUAL( expected, container );
}



void CFieldTest::InsertToVectorTest()
{
	//test storing into byte-buffer
	InsertToContainer<vector<int8_t>, int8_t>( -0x11 );
	InsertToContainer<vector<int8_t>, uint8_t>( -0x11 );
	InsertToContainer<vector<int8_t>, int16_t>( -0x1122 );
	InsertToContainer<vector<int8_t>, uint16_t>( -0x1122 );
	InsertToContainer<vector<int8_t>, int32_t>( -0x11223344 );
	InsertToContainer<vector<int8_t>, uint32_t>( -0x11223344 );
	InsertToContainer<vector<int8_t>, int64_t>( -0x1122334455667788 );
	InsertToContainer<vector<int8_t>, uint64_t>( -0x1122334455667788 );
	InsertToContainer<vector<int8_t>, float>( (float)123.456 );
	InsertToContainer<vector<int8_t>, double>( 123.456 );
	InsertToContainer<vector<int8_t>, long double>( 123.456 );

	//test storing into 2-byte-buffer
	InsertToContainer<vector<int16_t>, int16_t>( 0x1122 );
	InsertToContainer<vector<int16_t>, uint16_t>( 0x1122 );
	InsertToContainer<vector<int16_t>, int32_t>( 0x11223344 );
	InsertToContainer<vector<int16_t>, uint32_t>( 0x11223344 );
	InsertToContainer<vector<int16_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int16_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int16_t>, float>( (float)123.456 );
	InsertToContainer<vector<int16_t>, double>( 123.456 );
	InsertToContainer<vector<int16_t>, long double>( 123.456 );

	//test storing into 4-byte-buffer
	InsertToContainer<vector<int32_t>, int32_t>( 0x11223344 );
	InsertToContainer<vector<int32_t>, uint32_t>( 0x11223344 );
	InsertToContainer<vector<int32_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int32_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int32_t>, float>( (float)123.456 );
	InsertToContainer<vector<int32_t>, double>( 123.456 );
	InsertToContainer<vector<int32_t>, long double>( 123.456 );

	//test storing into 8-byte-buffer
	InsertToContainer<vector<int64_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int64_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<vector<int64_t>, double>( 123.456 );
	InsertToContainer<vector<int64_t>, long double>( 123.456 );

	//by design, these cases fail to compile
	//InsertToContainer<vector<int16_t>, int8_t>( -0x11 );
	//InsertToContainer<vector<int32_t>, int8_t>( -0x11 );
	//InsertToContainer<vector<int32_t>, int16_t>( -0x11 );
}


void CFieldTest::InsertToListTest()
{
	//test storing into byte-buffer
	InsertToContainer<list<int8_t>, int8_t>( -0x11 );
	InsertToContainer<list<int8_t>, uint8_t>( -0x11 );
	InsertToContainer<list<int8_t>, int16_t>( -0x1122 );
	InsertToContainer<list<int8_t>, uint16_t>( -0x1122 );
	InsertToContainer<list<int8_t>, int32_t>( -0x11223344 );
	InsertToContainer<list<int8_t>, uint32_t>( -0x11223344 );
	InsertToContainer<list<int8_t>, int64_t>( -0x1122334455667788 );
	InsertToContainer<list<int8_t>, uint64_t>( -0x1122334455667788 );
	InsertToContainer<list<int8_t>, float>( (float)123.456 );
	InsertToContainer<list<int8_t>, double>( 123.456 );
	InsertToContainer<list<int8_t>, long double>( 123.456 );

	//test storing into 2-byte-buffer
	InsertToContainer<list<int16_t>, int16_t>( 0x1122 );
	InsertToContainer<list<int16_t>, uint16_t>( 0x1122 );
	InsertToContainer<list<int16_t>, int32_t>( 0x11223344 );
	InsertToContainer<list<int16_t>, uint32_t>( 0x11223344 );
	InsertToContainer<list<int16_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<list<int16_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<list<int16_t>, float>( (float)123.456 );
	InsertToContainer<list<int16_t>, double>( 123.456 );
	InsertToContainer<list<int16_t>, long double>( 123.456 );

	//test storing into 4-byte-buffer
	InsertToContainer<list<int32_t>, int32_t>( 0x11223344 );
	InsertToContainer<list<int32_t>, uint32_t>( 0x11223344 );
	InsertToContainer<list<int32_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<list<int32_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<list<int32_t>, float>( (float)123.456 );
	InsertToContainer<list<int32_t>, double>( 123.456 );
	InsertToContainer<list<int32_t>, long double>( 123.456 );

	//test storing into 8-byte-buffer
	InsertToContainer<list<int64_t>, int64_t>( 0x1122334455667788 );
	InsertToContainer<list<int64_t>, uint64_t>( 0x1122334455667788 );
	InsertToContainer<list<int64_t>, double>( 123.456 );
	InsertToContainer<list<int64_t>, long double>( 123.456 );

	//by design, these cases fail to compile
	//InsertToContainer<list<int16_t>, int8_t>( -0x11 );
	//InsertToContainer<list<int32_t>, int8_t>( -0x11 );
	//InsertToContainer<list<int32_t>, int16_t>( -0x11 );
}




template<typename Buff, size_t nBytes, typename T>
static void CopyBytesWithPointer( T tData )
{
	//calc how big our array needs to be, but don't let it be zero
	enum
	{
		_div = nBytes / sizeof Buff,
		nCount = (_div > 0 ? _div : 1)
	};

	//expected output array; byte-copy t into it
	Buff aExpected[nCount] = {0};
	{
		const Field::_LangType<nBytes>::Result casted = static_cast<Field::_LangType<nBytes>::Result>( tData );
		memcpy( aExpected, &casted, min<size_t>(nBytes, sizeof casted)); //by defn, nBytes <= sizeof aExpected
		STATIC_CHECK( nBytes <= sizeof aExpected, Array_Too_Small );
	}

	//copy to a destination buffer
	Buff aBuff[nCount] = {0};
	Buff* const pNext = Field::Copy<nBytes>( aBuff, aBuff + nCount, tData );

	//make sure a copy into a too-small buffer throws
	CPPUNIT_ASSERT_THROW( Field::Copy<nBytes>( aBuff, aBuff + nCount -1, tData ), std::invalid_argument );

	//assert they're equal
	CPPUNIT_ASSERT( Equal( aExpected, aBuff ));

	//make sure our next insertion point follows the array
	CPPUNIT_ASSERT_EQUAL( aBuff + nCount, pNext );
}



void CFieldTest::CopyBytesWithPointerTest()
{
	//test interesting cases

	//size-n data into size-1 buffer, requesting size-n be used
	CopyBytesWithPointer<int8_t, 1, int8_t>( -0x11 );
	CopyBytesWithPointer<int8_t, 2, int16_t>( -0x1122 );
	CopyBytesWithPointer<int8_t, 4, int32_t>( -0x11223344 );
	CopyBytesWithPointer<int8_t, 8, int64_t>( -0x1122334455667788 );

	//size-n data into size-1 buffer, requesting >size-n be used
	CopyBytesWithPointer<int8_t, 2, int8_t>( -0x11 );
	CopyBytesWithPointer<int8_t, 4, int16_t>( -0x1122 );
	CopyBytesWithPointer<int8_t, 8, int32_t>( -0x11223344 );

	//size-n data into size-1 buffer, requesting <size-n be used
	CopyBytesWithPointer<int8_t, 1, int16_t>( -0x0011 );
	CopyBytesWithPointer<int8_t, 2, int32_t>( -0x00112233 );
	CopyBytesWithPointer<int8_t, 4, int64_t>( -0x0011223344556677 );

	//asking for more bytes than what's required
	CopyBytesWithPointer<uint16_t, 2, int16_t>( -0x1122 );
	CopyBytesWithPointer<uint16_t, 4, int16_t>( -0x1122 );
	CopyBytesWithPointer<uint16_t, 8, int16_t>( -0x1122 );

	//by design, these cases fail to compile
	//CopyBytesWithPointer<int16_t, 1, int8_t>( -0x11 );
	//CopyBytesWithPointer<int16_t, 1, int16_t>( -0x1122 );
}



//inserts data into a container using an iterator
template<class Cont, size_t nBytes, typename T>
static void CopyBytesWithIterator( T tData )
{
	typedef Cont::value_type Entry; //what the container holds
	enum { nCount = nBytes / sizeof Entry };

	//copy data into the container
	Cont container( nCount ); //pre-allocate entries so we don't overrun
	Cont::iterator iNext = Field::Copy<nBytes>( container.begin(), container.end(), tData );

	//make sure a copy into a too-small buffer throws
	CPPUNIT_ASSERT_THROW( Field::Copy<nBytes>( container.begin(), container.end() -1, tData ), std::invalid_argument );

	//assert we've got the right number of elements in the container
	CPPUNIT_ASSERT_EQUAL((size_t)nCount, container.size());

	//assert the contents are what we expect them to be
	const Field::_LangType<nBytes>::Result casted = static_cast<Field::_LangType<nBytes>::Result>( tData );
	const Cont expected((const Entry*)&casted, (const Entry*)(&casted+1));
	CPPUNIT_ASSERT_EQUAL( expected, container );

	//make sure our next insertion point is at the end of the container
	CPPUNIT_ASSERT( container.end() == iNext );
}



void CFieldTest::CopyBytesWithIteratorTest()
{
	//test interesting cases

	//size-n data into size-1 buffer, requesting size-n be used
	CopyBytesWithIterator<vector<int8_t>, 1, int8_t>( -0x11 );
	CopyBytesWithIterator<vector<int8_t>, 2, int16_t>( -0x1122 );
	CopyBytesWithIterator<vector<int8_t>, 4, int32_t>( -0x11223344 );
	CopyBytesWithIterator<vector<int8_t>, 8, int64_t>( -0x1122334455667788 );

	//size-n data into size-1 buffer, requesting >size-n be used
	CopyBytesWithIterator<vector<int8_t>, 2, int8_t>( -0x11 );
	CopyBytesWithIterator<vector<int8_t>, 4, int16_t>( -0x1122 );
	CopyBytesWithIterator<vector<int8_t>, 8, int32_t>( -0x11223344 );

	//size-n data into size-1 buffer, requesting <size-n be used
	CopyBytesWithIterator<vector<int8_t>, 1, int16_t>( -0x0011 );
	CopyBytesWithIterator<vector<int8_t>, 2, int32_t>( -0x00112233 );
	CopyBytesWithIterator<vector<int8_t>, 4, int64_t>( -0x0011223344556677 );

	//asking for more bytes than what's required
	CopyBytesWithIterator<vector<uint16_t>, 2, int16_t>( -0x1122 );
	CopyBytesWithIterator<vector<uint16_t>, 4, int16_t>( -0x1122 );
	CopyBytesWithIterator<vector<uint16_t>, 8, int16_t>( -0x1122 );

	//by design, these cases fail to compile
	//CopyBytesWithIterator<vector<int16_t>, 1, int8_t>( -0x11 );
	//CopyBytesWithIterator<vector<int16_t>, 1, int16_t>( -0x1122 );
}




template<class Cont, size_t nBytes, typename T>
static void InsertBytesToContainer( T tData )
{
	typedef Cont::value_type Entry; //what the container holds

	//number of entries we'll be writing into the container
	//even divisibility is checked by Insert 
	enum { nEntries = nBytes / sizeof Entry };

	//copy data into the container
	Cont container;
	Field::Insert<nBytes>( back_inserter(container), tData );

	//assert we've got the right number of elements in the container
	CPPUNIT_ASSERT_EQUAL( (size_t)nEntries, container.size());

	//assert we've got the right number of bytes
	CPPUNIT_ASSERT_EQUAL( nBytes, container.size() * sizeof Entry );

	//assert the contents are what we expect them to be
	typedef Field::_LangType<nBytes>::Result inter_t; //intermediate type using exactly nBytes
	const inter_t nInter = static_cast<inter_t>( tData ); //tData squished into nBytes
	const Entry* const pData = reinterpret_cast<const Entry*>(&nInter);
	const Cont expected( pData, pData + nEntries );
	CPPUNIT_ASSERT_EQUAL( expected, container );
}


void CFieldTest::InsertBytesToVectorTest()
{
	//test interesting cases

	//size-n data into size-1 buffer, requesting size-n be used
	InsertBytesToContainer<vector<int8_t>, 1, int8_t>( -0x11 );
	InsertBytesToContainer<vector<int8_t>, 2, int16_t>( -0x1122 );
	InsertBytesToContainer<vector<int8_t>, 4, int32_t>( -0x11223344 );
	InsertBytesToContainer<vector<int8_t>, 8, int64_t>( -0x1122334455667788 );

	//size-n data into size-1 buffer, requesting >size-n be used
	InsertBytesToContainer<vector<int8_t>, 2, int8_t>( -0x11 );
	InsertBytesToContainer<vector<int8_t>, 4, int16_t>( -0x1122 );
	InsertBytesToContainer<vector<int8_t>, 8, int32_t>( -0x11223344 );

	//size-n data into size-1 buffer, requesting <size-n be used
	InsertBytesToContainer<vector<int8_t>, 1, int16_t>( -0x0011 );
	InsertBytesToContainer<vector<int8_t>, 2, int32_t>( -0x00112233 );
	InsertBytesToContainer<vector<int8_t>, 4, int64_t>( -0x0011223344556677 );

	//asking for more bytes than what's required
	InsertBytesToContainer<vector<uint16_t>, 2, int16_t>( -0x1122 );
	InsertBytesToContainer<vector<uint16_t>, 4, int16_t>( -0x1122 );
	InsertBytesToContainer<vector<uint16_t>, 8, int16_t>( -0x1122 );

	//by design, these cases fail to compile
	//InsertBytesToContainer<vector<int16_t>, 1, int8_t>( -0x11 );
	//InsertBytesToContainer<vector<int16_t>, 1, int16_t>( -0x1122 );
}



//extracts data from an array
template<typename Buff, typename T>
static void ExtractBasicType( T tData )
{
	//calc how big our array needs to be, but don't let it be zero
	enum
	{
		_div = sizeof T / sizeof Buff,
		nCount = (_div > 0 ? _div : 1)
	};

	//setup our input array; byte-copy t into it
	Buff aInput[nCount];
	memcpy( aInput, &tData, sizeof tData ); //by defn, sizeof tData <= sizeof aInput
	STATIC_CHECK( sizeof tData <= sizeof aInput, Array_Too_Small );

	//read from the array into a local variable
	T tValueRead;
	Field::Extract( tValueRead, aInput, aInput + sizeof aInput );

	//assert we got out what we put in
	CPPUNIT_ASSERT_EQUAL( tData, tValueRead );
}


void CFieldTest::ExtractBasicTypeTest()
{
	//purposely use signed buffers, to catch signing issues

	//test reading from byte-buffer
	ExtractBasicType<int8_t, int8_t>( -0x11 );


	//test storing into byte-buffer
	CopyWithPointer<int8_t, int8_t>( -0x11 );
	CopyWithPointer<int8_t, uint8_t>( -0x11 );
	CopyWithPointer<int8_t, int16_t>( -0x1122 );
	CopyWithPointer<int8_t, uint16_t>( -0x1122 );
	CopyWithPointer<int8_t, int32_t>( -0x11223344 );
	CopyWithPointer<int8_t, uint32_t>( -0x11223344 );
	CopyWithPointer<int8_t, int64_t>( -0x1122334455667788 );
	CopyWithPointer<int8_t, uint64_t>( -0x1122334455667788 );
	CopyWithPointer<int8_t, float>( (float)123.456 );
	CopyWithPointer<int8_t, double>( 123.456 );
	CopyWithPointer<int8_t, long double>( 123.456 );

	//test storing into 2-byte-buffer
	CopyWithPointer<int16_t, int16_t>( 0x1122 );
	CopyWithPointer<int16_t, uint16_t>( 0x1122 );
	CopyWithPointer<int16_t, int32_t>( 0x11223344 );
	CopyWithPointer<int16_t, uint32_t>( 0x11223344 );
	CopyWithPointer<int16_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int16_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int16_t, float>( (float)123.456 );
	CopyWithPointer<int16_t, double>( 123.456 );
	CopyWithPointer<int16_t, long double>( 123.456 );

	//test storing into 4-byte-buffer
	CopyWithPointer<int32_t, int32_t>( 0x11223344 );
	CopyWithPointer<int32_t, uint32_t>( 0x11223344 );
	CopyWithPointer<int32_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int32_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int32_t, float>( (float)123.456 );
	CopyWithPointer<int32_t, double>( 123.456 );
	CopyWithPointer<int32_t, long double>( 123.456 );

	//test storing into 8-byte-buffer
	CopyWithPointer<int64_t, int64_t>( 0x1122334455667788 );
	CopyWithPointer<int64_t, uint64_t>( 0x1122334455667788 );
	CopyWithPointer<int64_t, double>( 123.456 );
	CopyWithPointer<int64_t, long double>( 123.456 );

	//by design, these cases fail to compile
	//CopyWithPointer<int16_t, int8_t>( -0x11 );
	//CopyWithPointer<int32_t, int8_t>( -0x11 );
	//CopyWithPointer<int32_t, int16_t>( -0x11 );
}


