/******************************************************************************
 * CLASSIFICATION: UNLASSIFIED//FOUO>
 *
 * Copyright(C) WebSec Corporation
 *
 *****************************************************************************/
/**
 * @file time_CE.h
 * Header file the CE time functions.
 */

///////////////////////////////////////////////////////////////////////////////
// pragmas
///////////////////////////////////////////////////////////////////////////////
#pragma once

 
#include <windows.h>

// These pragmas help Studio to build for some CE platforms - use as necessary
#pragma comment(linker, "/nodefaultlib:libc.lib")
#pragma comment(linker, "/nodefaultlib:libcd.lib")

#ifdef UNDER_CE
#pragma comment(lib, "secchk.lib")
#endif

// Number of clock ticks per second. Used by the clock() function.
#define CLOCKS_PER_SEC		1000

// Macro for CLK_PER_SEC
#define CLK_TICK			CLOCKS_PER_SEC

// structure definitions
#ifndef _TM_DEFINED
struct tm {
    int tm_sec;			// seconds
    int tm_min;			// minutes
    int tm_hour;		// hours
    int tm_mday;		// day of the month
    int tm_mon;			// month
    int tm_year;		// years since 1900 (from 0)
    int tm_wday;		// days since Sunday (from 0)
    int tm_yday;		// days since Jan 1
    int tm_isdst;		// daylight savings (summertime) flag
    };
#define _TM_DEFINED
#endif

// typedefs
typedef DWORD clock_t_ce;

// since time_t is already defined in CE as a ULONG, we need a new definition
typedef LONGLONG time_t_ce;

static struct tm tmbuf;

#ifdef __cplusplus
extern "C" 
{
#endif

/**
 * Convert tm to a string in the format "Www Mmm dd hh:mm:ss yyyy", 
 * where Www is the weekday, Mmm the month in letters, dd the day 
 * of the month, hh:mm:ss the time, and yyyy the year. The string 
 * is followed by a newline and a terminating null character, 
 * conforming a total of 26 characters. 
 *
 * @param tmptr A tm structure containing the time that will be
 *              converted.
 *
 * @return A char pointer containing the converted time.
 */
char *asctime_ce(const struct tm* tmptr);

/**
 * This method returns the number of clock ticks since the start
 * of the process.
 *
 * @returns A clock_t_ce (LONGLONG) containing the number of clock ticks.
 *
 * @notes This differs from the standard clock since GetTickCount is the
 *        number of milliseconds since system startup, not process start.
 *        This will also rollover after 49.7 days of continuous system runtime.
 */
clock_t_ce clock_ce(void);

/**
 * This method converts a time_t_ce value to a string in the same
 * format as asctime.
 *
 * @param timer A time_t_ce const value to be converted.
 *
 * @return A char pointer containing the converted time.
 */
char* ctime_ce(const time_t_ce* timer);

/**
 * This method returns the difference in seconds between the two times passed in.
 *
 * @param timer2 A time_t_ce value to use.
 * @param timer1 A time_t_ce value to use.
 *
 * @return A double containing the difference in secondes between the two times.
 */
double difftime_ce(time_t_ce timer2, time_t_ce timer1);

/**
 * Convert a time_t value to a tm structure as UTC time.
 *
 * @param timer A time_t_ce pointer containing the value to convert.
 *
 * @return A pointer to a tm structure that holds the converted time.
 */
struct tm* gmtime_ce(const time_t_ce* timer);

/**
 * Convert a time_t value to a tm structure as local time. (Reentrant version)
 *
 * @param timer A time_t_ce pointer containing the value to convert.
 *
 * @return A pointer to a tm structure that holds the converted time.
 */
struct tm* localtime_ce(const time_t_ce* timer);

/**
 * This method converts the local time to a calendar date.
 *
 * @param A pointer to a tm structure containing the local time.
 *
 * @return A time_t_ce value containing the calendar date.
 *
 * @notes time_t represents seconds since midnight January 1, 1970 UTC 
 *        (coordinated universal time) in 32-bits Win32 FILETIME structure is 64-bit,
 *        which represents the number of 100-nanosecond (hns) intervals since 
 *        January 1, 1601 UTC (coordinate universal time) the time difference
 *        between midnight January 1, 1970 and midnight January 1, 1601 is 11644473600 seconds
 */
time_t_ce mktime_ce(struct tm *tptr);

/**
 *
 * This method gets the current system time and converts it to seconds since
 * 1/1/1970. It stores the seconds value in tloc is not a NULL pointer and
 * then returns the seconds value.
 *
 * @param tloc A poitner to a time_t_ce which will hold the seconds
 *
 * @return A time_t_ce value containing the seconds.
 */
time_t_ce time_ce(time_t_ce* timer);

/**
 * This method formats a time string.
 *
 * @param s A pointer to the output buffer, there should be at least maxs
 *          characters available at the address pointed to by s.
 * @param maxs The maximum number of characters to be copied into the
 *             output buffer, including the '\0' terminator.
 * @param f The format string. A percent sign (%) is used to indicate that
 *          the following character is a special format character. See the
 *          notes section for valid format characters.
 * @param t A pointer to a tm structure containing the time to format.
 *
 * @return The number of characters placed in the buffer, not including the
 * terminating \0, or zero if more than maxs characters were produced.
 *
 * @notes The following are valid format characters:
 *
 *              %A      full weekday name (Monday)
 *              %a      abbreviated weekday name (Mon)
 *              %B      full month name (January)
 *              %b      abbreviated month name (Jan)
 *              %c      standard date and time representation
 *              %d      day-of-month (01-31)
 *              %H      hour (24 hour clock) (00-23)
 *              %I      hour (12 hour clock) (01-12)
 *              %j      day-of-year (001-366)
 *              %M      minute (00-59)
 *              %m      month (01-12)
 *              %p      local equivalent of AM or PM
 *              %S      second (00-59)
 *              %U      week-of-year, first day sunday (00-53)
 *              %W      week-of-year, first day monday (00-53)
 *              %w      weekday (0-6, sunday is 0)
 *              %X      standard time representation
 *              %x      standard date representation
 *              %Y      year with century
 *              %y      year without century (00-99)
 *              %Z      timezone name
 *              %%      percent sign
 *
 *      the standard date string is equivalent to:
 *
 *          %a %b %d %Y
 *
 *      the standard time string is equivalent to:
 *
 *          %H:%M:%S
 *
 *      the standard date and time string is equivalent to:
 *
 *          %a %b %d %H:%M:%S %Y
 *
 */
size_t strftime_ce(char *s, size_t maxs, const char *f, const struct tm *t);

#ifdef __cplusplus
}
#endif
